﻿// ImageShatterDemo - Using Papervision 2.0 Beta - Revision 753
// Causes an image to break into multiple planes which then fly away from there home location, then pause and fly back to there home location.
//
// released under MIT License
// http://www.opensource.org/licenses/mit-license.php
//
//@author		Devin Reimer
//@version		1.0.0
//@website 		http://www.permaven.com, http://www.almostlogical.com


//Copyright (c) 2008 Devin Reimer

/*
Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
*/

package com
{
	import flash.display.Sprite;
    import flash.events.Event;
	import flash.events.TimerEvent;
	import flash.display.BitmapData;
	import flash.geom.Rectangle;
	import flash.geom.Point;
	import flash.utils.Timer;

	import caurina.transitions.Tweener;
	
	import org.papervision3d.cameras.Camera3D;
	import org.papervision3d.materials.BitmapMaterial;
    import org.papervision3d.materials.utils.MaterialsList;
	import org.papervision3d.objects.primitives.Plane;
    import org.papervision3d.render.BasicRenderEngine;
    import org.papervision3d.scenes.Scene3D;
    import org.papervision3d.view.Viewport3D;
	
	public class ImageShatterDemo extends Sprite
	{
		private var viewport:Viewport3D;
		private var scene:Scene3D;
		private var camera:Camera3D;
		private var renderer:BasicRenderEngine;
		
		private static const TRIANGLES_PER_PLANE:Number=2; //num of triangles used per plane
		private static const MAIN_IMAGE_WIDTH:Number = 675;
		private static const MAIN_IMAGE_HEIGHT:Number = 225;
	
		public var numHorizPiece:Number=12*2; //num of pieces the image will be broken into horizontally
		public var numVerticalPiece:Number=4*2; //num of pieces the image will be broken into vertically
		public var completeCounter:Number=0;
		public var totalPieces = numHorizPiece * numVerticalPiece;  //total number of pieces the image will be broken in to
		
		public var piecesArray:Array = new Array();
		
		public function ImageShatterDemo()
		{
			stage.quality = "MEDIUM"; //Set the stage quality to low (for speed)
			
			startupTasks();
			beginRender();

			//addChild(new FPS());   //add when need to display FPS
		}
		
		public function startupTasks()
		{
			initPapervision3D();
			setupEnvironment();
		}
		
		private function initPapervision3D():void
		{ 
			viewport = new Viewport3D();
			addChild(viewport);
			
			scene = new Scene3D();
			camera = new Camera3D();  
			camera.z=0;
			camera.y=0;
			camera.x=250;
			
			camera.rotationY=-90; //this is set to -90 because this was created prior to major camera3D change 
			camera.zoom=10;
			camera.focus=100;
			
			renderer = new BasicRenderEngine(); 
       }
	   
	   private function beginRender():void{
          //calls the render function every frame
         	addEventListener(Event.ENTER_FRAME, render);
       }

       private function render(e:Event):void
	   { 
			renderer.renderScene(scene, camera, viewport);   
	   }
	   
	   public function setupEnvironment():void
	   {
		  createImagePieces();
	   }
	   
	   public function createImagePieces()
	   {
		    var cutImageWidth:Number=MAIN_IMAGE_WIDTH/numHorizPiece;
		    var cutImageHeight:Number=MAIN_IMAGE_HEIGHT/numVerticalPiece;
			var bitmapMaterialArray:Array = new Array();
			var piece:Plane;
			
			bitmapMaterialArray = cutUpImage(new MainImage(MAIN_IMAGE_WIDTH,MAIN_IMAGE_HEIGHT),cutImageWidth,cutImageHeight);
		
			piecesArray = new Array();
			
			//create all the needed pieces and place them in random positions, with random rotations;
			for (var t=0; t<numVerticalPiece; t++)
			{
				for (var n=0; n<numHorizPiece; n++)
				{
					piece = new Plane(bitmapMaterialArray[n + (t*numHorizPiece)],cutImageWidth,cutImageHeight,TRIANGLES_PER_PLANE,TRIANGLES_PER_PLANE);
					piece.rotationX=Math.random()*360;
					piece.rotationY=Math.random()*360;
					piece.rotationZ=Math.random()*360;
					
					piece.x = (Math.random()*400)-200;
					piece.z = (Math.random()*800) - 400;
					
					piecesArray.push(piece);
					scene.addChild(piece);
				}
	   		}	
			
			createForwardMovingTweens();
	   }
	   
	   //create tweens to move all image pieces to their home position
	   public function createForwardMovingTweens()
	   {
		   var verticalPos:int=0; //vertical pieces
		   var horizontalPos:int=0; //horizontal pieces
			
		   var cutImageWidth:Number=MAIN_IMAGE_WIDTH/numHorizPiece;
		   var cutImageHeight:Number=MAIN_IMAGE_HEIGHT/numVerticalPiece;
			
		   for (var counter:int = 0; counter<piecesArray.length; counter++)
		   {
			   if (horizontalPos>=numHorizPiece)
			   {
				 horizontalPos=0;
				 verticalPos++;
			   }
			   Tweener.addTween(piecesArray[counter], { x:-1000, y:-verticalPos*cutImageHeight + ((numVerticalPiece/2)*cutImageHeight), z:horizontalPos*cutImageWidth - (((numHorizPiece/2)-1)*cutImageWidth), rotationY:270, rotationX:0, rotationZ:0, time:10, onComplete: homePositionReached,  transition:"easeInQuart" } );
			   horizontalPos++;
		   }
	   }
	   
	   
	   public function homePositionReached()
	   {
		    var explodeTimer:Timer;
		   	completeCounter++;
			
			if (completeCounter>=totalPieces) //ones all images have reached their home position set timer for image to explode
			{
				explodeTimer = new Timer(2500,1);
				explodeTimer.addEventListener(TimerEvent.TIMER, explode);
				explodeTimer.start();
			}  
	   }
	   
	   //tween the image pieces apart
	   public function explode(eventArgs:TimerEvent)
	   {   
			var finishedTimer:Timer = new Timer(5000,1);
			
			completeCounter=0;
			for (var t=0; t<piecesArray.length; t++)
			{
				Tweener.addTween(piecesArray[t], { x:(Math.random()*2000)+200, y:0, z: (Math.random()*1000) - 500, rotationY:Math.random()*360, rotationX:Math.random()*360, rotationZ:Math.random()*360, time:10, transition:"easeOutQuart" } );
			}
			finishedTimer.addEventListener(TimerEvent.TIMER, restart);
			finishedTimer.start();
	   }
	   
	   //restart animation from beginning
	   public function restart(evt:Event)
	   {
		   Tweener.removeAllTweens();
		   createForwardMovingTweens();
	   }
	
		//This cuts up the image into its pieces and runs that information as an array of BitmapMaterial Objects
		public function cutUpImage(imageData:BitmapData,cutImageWidth:Number,cutImageHeight:Number):Array
		{
			var bitmapMaterialArray:Array = new Array();
			var tempBitmapData:BitmapData;
			var counter:Number=0;
			
			for (var t=0; t<numVerticalPiece; t++)
			{    for (var n=0; n<numHorizPiece; n++)
				{
					 tempBitmapData = new BitmapData(cutImageWidth,cutImageHeight, false, 0x00CC44);
					 tempBitmapData.copyPixels(imageData, new Rectangle(cutImageWidth*n,cutImageHeight*t,cutImageWidth,cutImageHeight), new Point(0, 0));
					 
					 bitmapMaterialArray[n + (t*numHorizPiece)] = new BitmapMaterial(tempBitmapData);
				}
			}
			
			return bitmapMaterialArray;
		}
		
	}//end ImageShatterDemo Class
	
	
}//end package
